#
# address.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'amstd/must'
require 'amstd/bug'
require 'amstd/futils'


module EST

  class AddressEntry
    
    def initialize( addr, name, keys = [] )
      keys.must Array
      self.addr = addr
      self.name = name
      @props = keys
    end

    attr :addr

    def addr=( adr )
      @addr = adr.dup
    end

    attr :name

    def name=( nm )
      @name = nm.dup
    end

    def properties
      @props.dup
    end

    def properties=( arr )
      arr.must Array
      arr.each {|i| i.must String }
      @props = arr.dup
    end

    def add_property( prop )
      @props |= prop
    end

    def del_property( prop )
      @props.delete prop
    end

  end


  class AddressBook

    def initialize( fname )
      @filename = File.expand_path( fname )
      @whole = {}
      @prop_tbl = {}
      File.open( @filename ) do |f|
        parse( f )
      end

      @edit = @get_addr = nil
    end


    def parse( f )
      in_group = false

      while line = f.gets do
        line.strip!
        next if line == '%'

        addr = line
        line = f.gets ; line.chomp! ; line.strip! ; name = line
        line = f.gets ; line.chomp! ; line.strip! ; keys = line.split(',')

        ent = AddressEntry.new( addr, name, keys )
        @whole[ addr ] = ent
        keys.each do |k|
          k.strip!
          next if k.size == 0
          unless arr = @prop_tbl[k] then
            @prop_tbl[k] = arr = []
          end
          arr.push ent
        end
      end
    end


    def get_addr( &block )
      unless @get_addr then
        @get_addr = GetAddress.new( self )
      end
      return @get_addr.invoke( block )
    end


    def edit( &block )
      unless @edit then
        @edit = EditAddress.new( self )
      end
      return @edit.invoke( block )
    end

    
    def []( addr )
      return @whole[ addr ]
    end

    def store( addr, entry )
      if @whole.has_key? addr then
        raise ArgumentError, "same address '#{addr}' was registered"
      end
      @whole[addr] = entry
    end

    def each
      @whole.each{|k,ent| yield ent }
    end

    def values
      @whole.values
    end

    def keys
      @whole.keys
    end

    def properties
      a = @prop_tbl.keys
      a.sort!
      a
    end

    def new_property( prop )
      if @prop_tbl.has_key? prop then
        raise ArgumentError, "property '#{prop}' is already registered"
      end
      @prop_tbl[ prop ] = []
    end

    def lookup( keys )
      ret = {}
      keys.each do |k|
        if arr = @prop_tbl[k] then
          arr.each{|a| ret[a] = true }
        end
      end
      return ret.keys
    end

  end


end   # module EST
