#
# composer.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/strategy'
require 'amstd/must'


module EST

  class NewMailComposer < Strategy

    HEADER = [ :date, :subject, :from, :mimever, :ctype, :xmailer ]

    RC_DEFAULT = {
      'my-header' => {}
    }

    def get
      port = CORE.mkmail( CORE.mailbox.draft ){|wp|
        @rc['headers'].each do |n,v|
          wp << n << ': '
          wp.puts eval(v)
        end
        wp.puts
      }

      port
    end


    private

    def date
      'Date' + ::TMail.time2str( Time.now )
    end

    def subject
      'Subject: '
    end

    def from
      'From: ' +
        (@rc['from-address'] || ENV['USER'] + '@' + ENV['HOSTNAME'])
    end

    def mimever
      'MIME-Version: 1.0'
    end

    def ctype
      'Content-Type: text/plain; charset=iso-2022-jp'
    end

    def xmailer
      "X-Mailer: mailest version #{Version}"
    end
  
  end


  class ReplyMailComposer < NewMailComposer

    HEADER = NewMailComposer::HEADER.dup
    HEADER.concat [ :to, :subject, :inreplyto, :references ]
    HEADER.uniq!

    tmp = [
      [ 'To',          %|replyto_to(original)|      ],
      [ 'Subject',     %|replyto_subject(original)| ],
      [ 'In-Reply-To', %|original.msgid|            ],
      [ 'References',  %|original.msgid|            ]
    ]

    RC_DEFAULT = {
      # 'header' => NewMailComposer::RC_DEFAULT['header'].dup.update(tmp),
      'cite-prifix' => '> ',
      'body'        => %(\n  In mail "%s"\n  %s wrote:\n\n)
    }

    def get( original )
      case original
      when Mail
        ;
      when TMail::Port
        original = Mail.new( original )
      else
        original.must Mail, TMail::Port
      end

      port = CORE.mkmail( CORE.mailbox.draft ){|wp|
        @rc.each do |n,v|
          wp.puts( n + ': ' + eval(v) )
        end
        wp.puts
        body( wp, original )
      }

      port
    end


    private

    REPLYTO_TO_HEADER = %w( reply-to from resent-reply-to resent-from )

    def to( original )
      REPLYTO_TO_HEADER.each do |n|
        if hf = original[n] then
          return 'To: ' + hf.body
        end
      end

      'To: '
    end
    
    def subject( original )
      sbj = original.subject('')
      sbj =  'Re: ' +
        if /\A\s*(?:\[[^\]]+\])?\s*(?:re\:\s*)*(\S.*)\Z/i === sbj
        then $1
        else sbj
        end

      sbj
    end

    def inreplyto( original )
      "In-Reply-To: #{original.msgid}"
    end

    def references
      "References: #{original.msgid}"
    end

    
    def body( wp, mail )
      tmp = @rc['body-arg']
      str = case tmp
            when Array  then sprintf( @rc['body'], *tmp )
            when String then sprintf( @rc['body'], tmp )
            else
              tmp.must Array, String
            end
      wp << str

      pre = @rc['cite-prefix']
      line = nil
      mail.each {|line| wp << pre << line }
      wp << "\n" unless /[\r\n]\z/ === line
    end

  end


  class ForwardMailComposer < Strategy

    RC_DEFAULT = NewMailComposer::RC_DEFAULT.dup

    def get( original )
      port = CORE.mkmail( CORE.mailbox.draft ){|wp|
        NEW_HEADER.each do |n,v|
          wp << n << ': '
          wp.puts eval(v)
        end
        if hf = original['to'] then
          if s = hf.addrs.join(', ') then
            wp << 'Resent-To: '
            wp.puts s
          end
        end
        if hf = original['from'] then
          if s = hf.addrs.join(', ') then
            wp << 'Resent-From: '
            wp.puts s
          end
        end
        wp.puts
        wp.puts '---------- forwarded mail:'
        original.body.each {|line| wp.puts line }
        wp.puts '---------- end forwarded mail'
      }

      port
    end

  end


  class SendingMailComposer

    RC_DEFAULT = {}

    def get( mail )
      mail.to_s
    end

  end

end
